#include <iostream>
#include <fcntl.h>
#include <libdnf5/base/base.hpp>
#include <libdnf5/common/exception.hpp>
#include <libdnf5/plugin/iplugin.hpp>
#include <libdnf5/rpm/package_query.hpp>
#include <libdnf5/rpm/package_sack.hpp>
#include <libdnf5/utils/bgettext/bgettext-mark-domain.h>
#include <unistd.h>


using namespace libdnf5;

namespace {

constexpr const char * PLUGIN_NAME = "fiflock";
constexpr plugin::Version PLUGIN_VERSION{0, 1, 0};

constexpr const char * attrs[]{"author.name", "author.email", "description", nullptr};
constexpr const char * attrs_value[]{
    "Philippe Troin", "phil@fifi.org", "Prevents fif packages from being upgraded to higher stock versions."};


class FifLock : public plugin::IPlugin {
public:
    FifLock(libdnf5::plugin::IPluginData & data, libdnf5::ConfigParser &) : IPlugin(data) {}
    virtual ~FifLock() = default;

    PluginAPIVersion get_api_version() const noexcept override { return PLUGIN_API_VERSION; }

    const char * get_name() const noexcept override { return PLUGIN_NAME; }

    plugin::Version get_version() const noexcept override { return PLUGIN_VERSION; }

    const char * const * get_attributes() const noexcept override { return attrs; }

    const char * get_attribute(const char * attribute) const noexcept override {
	for (size_t i = 0; attrs[i]; ++i) {
	    if (std::strcmp(attribute, attrs[i]) == 0) {
		return attrs_value[i];
	    }
	}
	return nullptr;
    }

    void repos_loaded() override;
};


void FifLock::repos_loaded() {
  auto & base = get_base();
  auto logger = base.get_logger();
  const auto & full_pkg_sack = base.get_rpm_package_sack();

  auto fif_packages_query = libdnf5::rpm::PackageQuery(base);
  fif_packages_query.filter_release("*.fif[0-9]*", libdnf5::sack::QueryCmp::GLOB);

  auto fif_excluded_query = libdnf5::rpm::PackageQuery(base);
  fif_excluded_query.filter_available();
  fif_excluded_query.filter_name(fif_packages_query, libdnf5::sack::QueryCmp::EQ);
  fif_excluded_query.filter_nevra(fif_packages_query, libdnf5::sack::QueryCmp::NEQ);

  full_pkg_sack.get()->add_user_excludes(fif_excluded_query);

  if (fif_excluded_query.size()) {
    std::string msg = std::to_string(fif_excluded_query.size()) + " packages excluded by fiflock";
    std::cerr << msg << std::endl;
    logger->notice(msg);
  }
}

}  // namespace

PluginAPIVersion libdnf_plugin_get_api_version(void) {
    return PLUGIN_API_VERSION;
}

const char * libdnf_plugin_get_name(void) {
    return PLUGIN_NAME;
}

plugin::Version libdnf_plugin_get_version(void) {
    return PLUGIN_VERSION;
}

plugin::IPlugin * libdnf_plugin_new_instance(
    [[maybe_unused]] LibraryVersion library_version,
    libdnf5::plugin::IPluginData & data,
    libdnf5::ConfigParser & parser) try {
    return new FifLock(data, parser);
} catch (...) {
    return nullptr;
}

void libdnf_plugin_delete_instance(plugin::IPlugin * plugin_object) {
    delete plugin_object;
}
